#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
# TestName documentation build configuration file, created by
# sphinx-quickstart on Fri Mar 23 11:32:48 2018.
#
# This file is execfile()d with the current directory set to its
# containing dir.
#
# Note that not all possible configuration values are present in this
# autogenerated file.
#
# Check out http://www.sphinx-doc.org/en/stable/config.html for further information
#
# All configuration values have a default; values that are commented out
# serve to show the default.

# If extensions (or modules to document with autodoc) are in another directory,
# add these directories to sys.path here. If the directory is relative to the
# documentation root, use os.path.abspath to make it absolute, like shown here.
#
# import os
# import sys
# sys.path.insert(0, os.path.abspath('.'))


# -- General configuration ------------------------------------------------

# If your documentation needs a minimal Sphinx version, state it here.
#
# needs_sphinx = '1.0'

# Add any Sphinx extension module names here, as strings. They can be
# extensions coming with Sphinx (named 'sphinx.ext.*') or your custom
# ones.
import os

extensions = ['sphinx.ext.mathjax',
              'sphinx.ext.ifconfig',
              'sphinx.ext.autodoc',
              'numpydoc']

# Add any paths that contain templates here, relative to this directory.
templates_path = ['_templates']

# The suffix(es) of source filenames.
# You can specify multiple suffix as a list of string:
#
# source_suffix = ['.rst', '.md']
source_suffix = '.rst'

# The encoding of source files.
#
source_encoding = 'latin-1'

# The master toctree document.
master_doc = 'index'

# General information about the project.
import datetime

now = datetime.datetime.now()

project = 'Component'
copyright = str(now.year) + ', LCM'
author = 'LCM'

# The version info for the project you're documenting, acts as replacement for
# |version| and |release|, also used in various other places throughout the
# built documents.
#
version_file = os.path.join(os.curdir, '__version__.txt')  # Manual: Major.Minor
revision_file = os.path.join(os.curdir, '__revision__.txt')  # Automatic: Revision

if os.path.isfile(version_file):
    'version file may not exist'
    with open(version_file, 'r') as fh:
        major_minor_version = fh.read()
else:
    '''dummy version'''
    major_minor_version = '0.0'

if os.path.isfile(revision_file):
    'revision file may not exist'
    with open(revision_file, 'r') as fh:
        revision = fh.read()
else:
    '''dummy revision'''
    revision = '0'

# The short X.Y version.
# version = '0'
version = major_minor_version

# The full version, including alpha/beta/rc tags.
# release = '1'
release = revision

# The language for content autogenerated by Sphinx. Refer to documentation
# for a list of supported languages.
#
# This is also used if you do content translation via gettext catalogs.
# Usually you set "language" from the command line for these cases.
language = None

# List of patterns, relative to source directory, that match files and
# directories to ignore when looking for source files.
# This patterns also effect to html_static_path and html_extra_path
exclude_patterns = []

# The name of the Pygments (syntax highlighting) style to use.
pygments_style = 'sphinx'

# If true, `todo` and `todoList` produce output, else they produce nothing.
todo_include_todos = False

rst_prolog = """
.. |sym| replace:: SyMSpace
.. role:: gray
.. role:: guibutton
.. |clearfloat|  raw:: html

    <div class="clearer"></div>
	
"""

# -- Options for HTML output ----------------------------------------------

# The theme to use for HTML and HTML Help pages.  See the documentation for
# a list of builtin themes.
#
html_theme = 'classic'

# Theme options are theme-specific and customize the look and feel of a theme
# further.  For a list of options available for each theme, see the
# documentation.
#
lcmblue = "#00ADEF"
lcmgray2 = "#D9DADB"
lcmgray4 = "#B1B2B4"
lcmgray7 = "#6F7072"
gray1 = "rgb(95%,95%,95%)"

html_theme_options = {
    "sidebarbgcolor": lcmgray4,
    "sidebartextcolor": "white",
    "sidebarlinkcolor": "white",
    "relbarbgcolor": lcmgray7,
    "bgcolor": "white",
    "linkcolor": lcmgray7,
    "visitedlinkcolor": lcmgray7,
    "headbgcolor": lcmblue,
    "headtextcolor": "white",
    "headlinkcolor": "white",
    "codebgcolor": gray1,
    "footerbgcolor": "black"
}
# Add any paths that contain custom static files (such as style sheets) here,
# relative to this directory. They are copied after the builtin static files,
# so a file named "default.css" will overwrite the builtin "default.css".
html_static_path = ['rst/_static']


# this is used to add an additional stylefile (not only overwrite an existing)
def setup(app):
    try:
        app.add_css_file('custom.css')
    except AttributeError:
        '''deprecated at sphinx=1.8.0'''
        app.add_stylesheet('custom.css')


# The name for this set of Sphinx documents.
# "<project> v<release> documentation" by default.
#
html_title = project + " v" + version + "." + release + " documentation"
# html_title = ''

# Custom sidebar templates, must be a dictionary that maps document names
# to template names.
#
# This is required for the alabaster theme
# refs: http://alabaster.readthedocs.io/en/latest/installation.html#sidebars
'''
html_sidebars = {
    '**': [
        'about.html',
        'navigation.html',
        'relations.html',  # needs 'show_related': True theme option to display
        'searchbox.html',
        'donate.html',
    ]
}
'''
html_sidebars = {'**': []}

# To prevent copying the source code set to False
# This limits the search capability of the javascript!
html_copy_source = False

# To prevent creation of an index set to False
html_use_index = False

# If not None, a 'Last updated on:' timestamp is inserted at every page
# bottom, using the given strftime format.
# The empty string is equivalent to '%b %d, %Y'.

html_last_updated_fmt = '%b %d, %Y'
