#!/usr/bin/python
# -*- coding: <UTF-8> -*-

"""
Python Version : 3.5

$LastChangedRevision: 5146 $
$LastChangedDate:: 2021-11-03 08:22:53 +0100#$

This file is part of SyMSpace.

@author: Armin Fohler

Copyright (c) 2017, Linz Center of Mechatronics GmbH (LCM) http://www.lcm.at/
All rights reserved.
"""

import numpy as np
import matplotlib.pyplot as plt
from scipy import optimize
import scipy.linalg
from math import atan


class MaterialFunctions:

    # def calc_loss_coefficients_LSQ(material_dictionary):
    #
    #     material['khfe'] = []
    #     material['kefe'] = []
    #     f0 = 50
    #     B0 = 1.5
    #     mu_0 = 4 * np.pi * 1e-7
    #
    #     for i in range(len(material_dictionary['Frequency']['value'])):
    #         f = material_dictionary['Frequency']['value'][i]
    #         loss = np.transpose(material_dictionary['Specific total loss']['value'][i])
    #
    #         # B = J + mu_0*H
    #         B = material_dictionary['Magnetic polarization']['value'][i] / 1000.0 + mu_0 * material_dictionary['Magnetic field strength']['value'][i]
    #         print('B = ', B)
    #
    #         # fit Steinmetz-Formula:
    #         steinmetz = lambda kfe, x, y: kfe[0] * pow(x / B0, 2.0) * (y / f0) + kfe[1] * pow(x / B0, 2.0) * pow(y / f0, 2.0)
    #         errfunc = lambda kfe, x, y, z: steinmetz(kfe, x, y) - z  # Distance to the target function
    #
    #         # fitfunc = lambda kfe, x, y: kfe[0] * y * pow(B0 / x, 2.0) + kfe[1] * y * pow(B0 / x, 2.0)
    #         # errfunc = lambda kfe, x, y, z: fitfunc(kfe, x, y) - z  # Distance to the target function
    #
    #         kfe0 = [0.0, 0.0]  # Initial parameter-guess
    #         kfe1, success = optimize.leastsq(errfunc, kfe0[:], args=(B, f, loss))
    #
    #         # plot if necessary
    #         time = np.linspace(B.min(), B.max(), 100)
    #         plt.plot(B, loss, "ro", time, steinmetz(kfe1, time, f), "r-")  # Plot of the data and the fit
    #
    #         # Legend the plot
    #         plt.title("Loss-Coefficients (khfe, kefe) fitted using Steinmetz-Formula: \n"
    #                   "khfe * pow(B / B0, 2.0) * (f / f0) + kefe * pow(B / B0, 2.0) * pow(f / f0, 2.0)")
    #         plt.xlabel("B []")
    #         plt.ylabel("Loss [um]")
    #         plt.legend(('Messdaten', 'Berechnete Werte'))
    #
    #         ax = plt.axes()
    #         plt.text(0.8, 0.07, 'khfe :  %.3f \n kefe:  %.3f ' % (float(kfe1[0]), float(kfe1[1])), fontsize=16,
    #                  horizontalalignment='center', verticalalignment='center', transform=ax.transAxes)
    #
    #         plt.show()
    #
    #         material['khfe'].append(float(kfe1[0]))
    #         material['kefe'].append(float(kfe1[1]))

    @staticmethod
    def calc_loss_coefficients_QR(material_dictionary):
        material = {}
        material['khfe'] = []
        material['kefe'] = []
        f0 = 50
        B0 = 1.5
        mu_0 = 4 * np.pi * 1e-7

        # the calculation for the coefficients has to be done at least for 2 frequencies, else the resulting matrix AtA
        # is not positive definite
        for i in range(len(material_dictionary['Frequency']['value'])-1):
            n = len(material_dictionary['Magnetic polarization']['value'][i]) + len(material_dictionary['Magnetic polarization']['value'][i + 1])
            n1 = len(material_dictionary['Magnetic polarization']['value'][i])

            loss = np.zeros((n, 1))
            A = np.zeros((n, 2))

            for k in range(2):
                f = material_dictionary['Frequency']['value'][i+k]

                # B = J + mu_0*H
                # B = material_dictionary['Magnetic polarization']['value'][i+k] / 1000.0 + mu_0 * material_dictionary['Magnetic field strength']['value'][i+k]
                B = [a + b for a, b in zip([j / 1000.0 for j in material_dictionary['Magnetic polarization']['value'][i+k]], [h * mu_0 for h in material_dictionary['Magnetic field strength']['value'][i+k]])]

                for m in range(len(B)):
                    A[m+k*n1, 0] = f / f0
                    A[m+k*n1, 1] = pow(f / f0, 2.0)

                    loss[m+k*n1] = pow(B0 / B[m], 2.0) * material_dictionary['Specific total loss']['value'][i+k][m]
                    # print(material_dictionary['Specific total loss']['value'][i+k][m])
                    # print(B0)
                    # print(B[m])
                    # input('...')

            # QR decomposition of A (via R = Cholesky(AT*A) and Q = R-1*A)
            # A x = b with A = [fB^2    f^2B^2]; x = (khfe,  kefe); b = (P)

            # print('AtA = ', np.dot(np.transpose(A), A))
            R = scipy.linalg.cholesky(np.dot(np.transpose(A), A))
            # print('R = ', R)
            Q = np.dot(A, np.linalg.inv(R))
            # print('Q = ', Q)
            # print('loss = ', loss)
            z = np.dot(np.transpose(Q), loss)
            # print('z = ', z)

            kfe_qr = np.dot(np.linalg.inv(R), z)

            print('QR kfe = ', kfe_qr)

            # # plot if necessary
            # time = np.linspace(B.min(), B.max(), 100)
            # plt.plot(B, loss, "ro", time, steinmetz(kfe1, time, f), "r-")  # Plot of the data and the fit
            #
            # # Legend the plot
            # plt.title("Loss-Coefficients (khfe, kefe) fitted using Steinmetz-Formula: \n"
            #           "khfe * pow(B / B0, 2.0) * (f / f0) + kefe * pow(B / B0, 2.0) * pow(f / f0, 2.0)")
            # plt.xlabel("B []")
            # plt.ylabel("Loss [um]")
            # plt.legend(('Messdaten', 'Berechnete Werte'))
            #
            # ax = plt.axes()
            # plt.text(0.8, 0.07, 'khfe :  %.3f \n kefe:  %.3f ' % (float(kfe1[0]), float(kfe1[1])), fontsize=16,
            #          horizontalalignment='center', verticalalignment='center', transform=ax.transAxes)
            #
            # plt.show()

            material['khfe'].append(float(kfe_qr[0]))
            material['kefe'].append(float(kfe_qr[1]))

            return material
        
    ''' Calculate loss coefficients using the Moore-Penrose_pseudo_Inverse '''    
    @staticmethod
    def calc_loss_coefficients_MP(material_dictionary):
        material = {}
        material['khfe_F'] = {}
        material['kefe_F'] = {}
        f0 = 50
        B0 = 1.5
        mu_0 = 4 * np.pi * 1e-7

        # the calculation for the coefficients has to be done at least for 2 frequencies, else the resulting matrix AtA
        # is not positive definite
        for i in range(len(material_dictionary['Frequency']['value'])-1):
            n = len(material_dictionary['Magnetic flux density']['value'][i]) + len(material_dictionary['Magnetic flux density']['value'][i + 1])
            n1 = len(material_dictionary['Magnetic flux density']['value'][i])
            
            loss = np.zeros((n, 1))
            A = np.zeros((n, 2))
            
            for k in range(2):
                f = material_dictionary['Frequency']['value'][i+k]

                # B = J + mu_0*H
                # B = material_dictionary['Magnetic polarization']['value'][i+k] / 1000.0 + mu_0 * material_dictionary['Magnetic field strength']['value'][i+k]
                B = material_dictionary['Magnetic flux density']['value'][i+k]

                shift = 0
                for m in range(len(B)):
                    A[m+k*n1, 0] = f / f0
                    A[m+k*n1, 1] = pow(f / f0, 2.0)

                    if B[m] == 0:
                        loss[m + k * n1] = 0
                        if material_dictionary['Specific total loss']['value'][i+k][m] != 0:
                            shift = shift + 1
                    else:
                        loss[m+k*n1] = pow(B0 / B[m], 2.0) * material_dictionary['Specific total loss']['value'][i+k][m-shift]
                    # print(material_dictionary['Specific total loss']['value'][i+k][m])
                    # print(B0)
                    # print(B[m])
                    # input('...')

            del_row = []
            for t in range(len(loss)):
                if not loss[t] > 0:
                    del_row.append(t)

            A_del = np.delete(A, del_row, 0)

            loss_del = list(filter(lambda val: val > 0, loss))

            # short with Moore-Penrose-pseudoinverse:
            A_plus = np.linalg.pinv(A_del)
            # print('A_plus = ', A_plus)
            kfe_mp = np.dot(A_plus, loss_del)

            print('Moore Penrose kfe = ', kfe_mp)
            
            # # plot if necessary
            # time = np.linspace(B.min(), B.max(), 100)
            # plt.plot(B, loss, "ro", time, steinmetz(kfe1, time, f), "r-")  # Plot of the data and the fit
            #
            # # Legend the plot
            # plt.title("Loss-Coefficients (khfe, kefe) fitted using Steinmetz-Formula: \n"
            #           "khfe * pow(B / B0, 2.0) * (f / f0) + kefe * pow(B / B0, 2.0) * pow(f / f0, 2.0)")
            # plt.xlabel("B []")
            # plt.ylabel("Loss [um]")
            # plt.legend(('Messdaten', 'Berechnete Werte'))
            #
            # ax = plt.axes()
            # plt.text(0.8, 0.07, 'khfe :  %.3f \n kefe:  %.3f ' % (float(kfe1[0]), float(kfe1[1])), fontsize=16,
            #          horizontalalignment='center', verticalalignment='center', transform=ax.transAxes)
            #
            # plt.show()

            material['khfe_F'][str(material_dictionary['Frequency']['value'][i]) + '-' + str(material_dictionary['Frequency']['value'][i+1])] = float(kfe_mp[0])
            material['kefe_F'][str(material_dictionary['Frequency']['value'][i]) + '-' + str(material_dictionary['Frequency']['value'][i+1])] = float(kfe_mp[1])

        A_0 = []
        A_1 = []
        loss_list = []
        for i in range(len(material_dictionary['Frequency']['value'])):
            if material_dictionary['Frequency']['value'][i] <= 600:
                f = material_dictionary['Frequency']['value'][i]

                B = material_dictionary['Magnetic flux density']['value'][i]

                shift = 0
                for m in range(len(B)):
                    A_0.append(f / f0)
                    A_1.append(pow(f / f0, 2.0))

                    if B[m] == 0:
                        loss_list.append(0)
                        if material_dictionary['Specific total loss']['value'][i][m] != 0:
                            shift = shift + 1
                    else:
                        loss_list.append(pow(B0 / B[m], 2.0) * material_dictionary['Specific total loss']['value'][i][m - shift])

                #     # print(material_dictionary['Specific total loss']['value'][i+k][m])
                #     # print(B0)
                #     # print(B[m])
                #     # input('...')

                # short with Moore-Penrose-pseudoinverse:

        A = np.column_stack((A_0, A_1))

        del_row = []
        for t in range(len(loss_list)):
            if not loss_list[t] > 0:
                del_row.append(t)

        A_del = np.delete(A, del_row, 0)

        loss_del = list(filter(lambda val: val > 0, loss_list))

        loss = np.array(loss_del)

        A_plus = np.linalg.pinv(A_del)
        # print('A_plus = ', A_plus)
        kfe_mp = np.dot(A_plus, loss)

        print("kfe_mp = ", kfe_mp)
        material['khfe'] = float(kfe_mp[0])
        material['kefe'] = float(kfe_mp[1])

        return material


if __name__ == "__main__":
    # material_dictionary = {'name': 'Isovac270-35A',
    #                        'Frequency': {'unit': 'Hz', 'value': [50, 200]},
    #                        'Magnetic polarization': {'unit': 'mT', 'value': [[0, 150, 200, 250, 300, 350, 400, 450, 500, 550, 600, 650, 700, 750, 800, 850, 900, 1000, 1050, 1100, 1150, 1200, 1250, 1300, 1350, 1400, 1450, 1500, 1550, 1600, 1650, 1700], [0, 150, 200, 250, 300, 350, 400, 450, 500, 550, 600, 650, 700, 750, 800, 850, 900, 1000, 1050, 1100, 1150, 1200, 1250, 1300, 1350, 1400, 1450, 1500, 1550, 1600, 1650, 1700]]},
    #                        'Magnetic field strength': {'unit': 'A/m', 'value': [[0, 29, 33, 37, 41, 44, 48, 51, 55, 58, 61, 64, 67, 70, 73, 76, 80, 92, 99, 109, 120, 139, 170, 213, 290, 496, 931, 1635, 2634, 3951, 5589, 7454], [0, 31, 36, 42, 47, 51, 56, 61, 65, 69, 73, 77, 81, 85, 89, 93, 97, 108, 115, 122, 130, 145, 172, 213, 290, 497, 934, 1644, 2651, 3980, 5627, 7501]]},
    #                        'Specific total loss': {'unit': 'W/kg', 'value': [[0, 0.05, 0.08, 0.11, 0.15, 0.18, 0.22, 0.27, 0.31, 0.37, 0.42, 0.48, 0.55, 0.62, 0.69, 0.76, 0.84, 1.01, 1.1, 1.21, 1.32, 1.43, 1.56, 1.72, 1.9, 2.1, 2.3, 2.5, 2.69, 2.88, 3.06, 3.24], [0, 0.28, 0.46, 0.65, 0.86, 1.08, 1.32, 1.59, 1.88, 2.2, 2.56, 2.94, 3.35, 3.79, 4.26, 4.74, 5.26, 6.38, 6.99, 7.66, 8.37, 9.14, 9.97, 10.93, 12.09, 13.34, 14.58, 15.88, 17.28, 18.78, 20.36, 21.99]]}}
    material_dictionary = {'name': 'Isovac400-65A',
                           'Frequency': {'unit': 'Hz', 'value': [50, 60, 200, 400, 500]},
                           'Magnetic polarization': {'unit': 'mT', 'value': [
                               [0, 150, 200, 250, 300, 350, 400, 450, 500, 550, 600, 650, 700, 750, 800, 850, 900, 1000,
                                1050, 1100, 1150, 1200, 1250, 1300, 1350, 1400, 1450, 1500, 1550, 1600, 1650, 1700],
                               [0, 150, 200, 250, 300, 350, 400, 450, 500, 550, 600, 650, 700, 750, 800, 850, 900, 1000,
                                1050, 1100, 1150, 1200, 1250, 1300, 1350, 1400, 1450, 1500, 1550, 1600, 1650, 1700],
                               [0, 100, 150, 200, 250, 300, 350, 400, 450, 500, 550, 600, 650, 700, 750, 800, 850, 900,
                                1000, 1050, 1100, 1150, 1200, 1250, 1300, 1350, 1400, 1450, 1500, 1550, 1600, 1650, 1700],
                               [0, 100, 150, 200, 250, 300, 350, 400, 450, 500, 550, 600, 650, 700, 750, 800, 850, 900,
                                1000, 1050, 1100, 1150, 1200, 1250, 1300, 1350, 1400, 1450, 1500, 1550, 1600, 1650, 1700],
                               [0, 100, 150, 200, 250, 300, 350, 400, 450, 500, 550, 600, 650, 700, 750, 800, 850, 900,
                                1000, 1050, 1100, 1150, 1200, 1250, 1300, 1350, 1400, 1450, 1500]]},
                           'Magnetic field strength': {'unit': 'A/m', 'value': [
                               [0, 34, 39, 44, 48, 53, 58, 63, 67, 72, 77, 81, 86, 91, 96, 102, 108, 123, 132, 142, 155,
                                173, 198, 231, 281, 381, 586, 1004, 1740, 2829, 4269, 5957],
                               [0, 36, 41, 46, 51, 56, 61, 66, 70, 75, 79, 83, 88, 92, 97, 102, 107, 121, 128, 137, 147,
                                160, 180, 209, 256, 359, 576, 1014, 1777, 2911, 4419, 6196],
                               [0, 34, 41, 49, 56, 63, 70, 78, 85, 92, 99, 107, 114, 122, 130, 138, 147, 157, 178, 189,
                                201, 212, 227, 246, 267, 296, 378, 580, 1007, 1764, 2893, 4397, 6170],
                               [0, 39, 49, 58, 68, 78, 88, 99, 110, 121, 133, 145, 158, 172, 187, 203, 219, 237, 276,
                                297, 319, 341, 366, 395, 419, 434, 481, 627, 1012, 1763, 2907, 4420, 6195],
                               [0, 40, 51, 61, 72, 83, 95, 107, 120, 134, 148, 164, 180, 198, 216, 236, 257, 279, 329,
                                357, 386, 416, 442, 465, 506, 570, 574, 545, 1025]]},
                           'Specific total loss': {'unit': 'W/kg', 'value': [
                               [0.00, 0.06, 0.1, 0.15, 0.19, 0.24, 0.3, 0.36, 0.42, 0.5, 0.58, 0.66, 0.76, 0.85, 0.96,
                                1.07, 1.18, 1.43, 1.56, 1.7, 1.85, 2, 2.17, 2.35, 2.54, 2.75, 3, 3.25, 3.51, 3.76, 4.02, 4.29],
                               [0.00, 0.08, 0.13, 0.18, 0.24, 0.3, 0.37, 0.45, 0.53, 0.62, 0.72, 0.82, 0.94, 1.06, 1.19,
                                1.32, 1.46, 1.77, 1.94, 2.13, 2.32, 2.52, 2.74, 2.97, 3.23, 3.52, 3.84, 4.17, 4.49, 4.81, 5.13, 5.47],
                               [0.00, 0.14, 0.41, 0.69, 0.98, 1.29, 1.64, 2.02, 2.44, 2.92, 3.45, 4.04, 4.7, 5.42, 6.2,
                                7.05, 7.97, 8.96, 11.17, 12.4, 13.73, 15.16, 16.7, 18.38, 20.17, 22.08, 24.12, 26.32, 28.59, 30.89, 33.32, 36.01, 38.88],
                               [0.00, 0.42, 1.14, 1.89, 2.69, 3.56, 4.54, 5.65, 6.9, 8.34, 9.97, 11.82, 13.9, 16.22,
                                18.79, 21.63, 24.75, 28.16, 35.96, 40.38, 45.19, 50.41, 56.07, 62.21, 68.8, 75.8, 83.23, 91.13, 99.47, 108.32, 118.01, 128.79, 140.4],
                               [0.00, 0.57, 1.54, 2.54, 3.63, 4.84, 6.22, 7.79, 9.61, 11.72, 14.14, 16.9, 20.01, 23.47,
                                27.31, 31.52, 36.14, 41.27, 53.52, 60.78, 68.41, 76.21, 84.52, 93.7, 103.83, 114.78, 125.72, 136.6, 151.05]]},
                           'Magnetic flux density': {'unit': '', 'value': []}}

    mu_0 = 4 * np.pi * 1e-7
    B = []
    for j in range(len(material_dictionary['Magnetic polarization']['value'])):
        B.append([a + b for a, b in zip([j / 1000.0 for j in material_dictionary['Magnetic polarization']['value'][j]],
                               [h * mu_0 for h in material_dictionary['Magnetic field strength']['value'][j]])])
    # B = [a + b for a, b in zip([j / 1000.0 for j in material_dictionary['Magnetic polarization']['value'][0]],
    #                            [h * mu_0 for h in material_dictionary['Magnetic field strength']['value'][0]])]
    material_dictionary['Magnetic flux density']['value'] = B
    print(material_dictionary['Magnetic flux density'])

    #material_qr = MaterialFunctions.calc_loss_coefficients_QR(material_dictionary)
    material_mp = MaterialFunctions.calc_loss_coefficients_MP(material_dictionary)
    
    #print('material_qr: ', material_qr)
    print('material_mp: ', material_mp)
