# !/usr/bin/python
# -*- coding: <UTF-8> -*-

"""
Python Version : 3.6

$LastChangedRevision: 4810 $
$LastChangedDate:: 2021-09-10 16:32:33 +0200#$

This file is part of SyMSpace.

Copyright (c) 2021, Linz Center of Mechatronics GmbH (LCM) http://www.lcm.at/
All rights reserved.
"""
if __name__ == '__main__':
    import argparse
    from symspace.osutils.pathutils import CMDParser
    from symspace.wrapper.symwrapper import SymProject
    from symspace.utils.tree_manipulation import ModelDescription

    parser = argparse.ArgumentParser(description='Execute build script')
    parser.add_argument('mop_file', help='Test Mop-File')
    parser.add_argument('xls_file', help='Excel file')

    args = CMDParser.jepSysArgs()
    args = parser.parse_args(args)

    mop_file = CMDParser.jepArgPath(args.mop_file)
    xls_file = CMDParser.jepArgPath(args.xls_file)

    sym_prj = SymProject.new('Test', 'Test', mop_file)

    md = ModelDescription(sym_prj, index=0, log_level='WARNING')
    md.import_add_missing_containers = True
    md.import_add_missing_fields = True
    md.updateFromExcel(xls_file)

    error = sym_prj.getError()
    event_log = sym_prj.getEventLog()

    if md.errors > 0:
        raise Exception(md.getErrorString())
    if error:
        raise Exception(event_log)
    else:
        sym_prj.save()

    print("Script finished")
