#!/usr/bin/python
# -*- coding: <UTF-8> -*-

"""
Python Version : 3.6

$LastChangedRevision: 3049 $
$LastChangedDate:: 2020-04-29 15:33:07 +0200#$

This file is part of SyMSpace.

Copyright (c) 2020, Linz Center of Mechatronics GmbH (LCM) http://www.lcm.at/
All rights reserved.
"""
from PIL import Image, ImageChops


class ImageProcessing:
    def __init__(self, fname):
        self.file = fname
        self.image = Image.open(fname)

    def merge(self, limg, scale=0.5, pos=(1.0, 0.0)):
        """
        Merges an image (logo)

        Parameters
        ----------
        limg : ImageProcessing
            Image of logo

        scale : float
            Scale factor of logo with respect to image

        pos : tuple
            Position of logo
            (0.0, 0.0): upper left
            (1.0, 1.0): lower right
        """
        if isinstance(limg, ImageProcessing):
            img = limg.image
        else:
            img = Image.open(limg)

        '''resize logo'''
        ratio = max(img.size[0] / self.image.size[0], img.size[1] / self.image.size[1])
        size = (int(float(img.size[0]) * scale / ratio),
                int(float(img.size[1]) * scale / ratio))
        logo = img.resize(size)
        img_box = self.image.getbbox()
        logo_box = logo.getbbox()

        '''right bottom corner'''
        box = (int((img_box[2] - logo_box[2]) * pos[0]),
               int((img_box[3] - logo_box[3]) * pos[1]))

        # ToDo: improve this
        try:
            simage_trans = Image.new("RGBA", logo.size)
            simage_trans = Image.blend(simage_trans, logo, 1.0)
            self.image.paste(logo, box, simage_trans)
        except:
            self.image.paste(logo, box)

    def strip(self):
        """
        Removes border from image. Border color is taken from upper left pixel
        """
        im = self.image
        bg = Image.new(im.mode, im.size, im.getpixel((0, 0)))
        diff = ImageChops.difference(im, bg)
        diff = ImageChops.add(diff, diff, 2.0, -100)
        bbox = diff.getbbox()
        self.image = im.crop(bbox)

    def save(self, fname=None):
        if fname is None:
            fname = self.file
        self.image.save(fname)
